import sbt._
import Keys._
import java.net.URL

object Dependencies {
  val scalatest = "org.scalatest" % "scalatest_2.9.0" % "1.6.1" % "test"
  val specs = "org.scala-tools.testing" % "specs_2.9.0-1" % "1.6.8" % "test"
  val antiXML = "com.codecommit" %% "anti-xml" % "0.3-SNAPSHOT"
  val dispatch = "net.databinder" %% "dispatch-http" % "0.8.4"
  val unfiltered_filter = "net.databinder" %% "unfiltered-filter" % "0.4.1"
  val unfiltered_jetty = "net.databinder" %% "unfiltered-jetty" % "0.4.1"
  val unfiltered_scalate = "net.databinder" %% "unfiltered-scalate" % "0.4.1"
  val unfiltered_json = "net.databinder" %% "unfiltered-json" % "0.4.1"
  val unfiltered_spec = "net.databinder" %% "unfiltered-spec" % "0.4.1" % "test"
  val slf4jSimple = "org.slf4j" % "slf4j-simple" % "1.6.1"
  val salat = "com.novus" %% "salat-core" % "0.0.8-SNAPSHOT"
}

object BuildSettings {

  val buildOrganization = "org.w3"
  val buildVersion      = "0.1-SNAPSHOT"
  val buildScalaVersion = "2.9.0-1"

  val buildSettings = Defaults.defaultSettings ++ Seq (
    organization := buildOrganization,
    version      := buildVersion,
    scalaVersion := buildScalaVersion,
    parallelExecution in Test := false,
    scalacOptions ++= Seq("-deprecation", "-unchecked")
  )

}

object VirtualTrainerBuild extends Build {

  import Dependencies._
  import BuildSettings._

  val virtual_trainer_settings =
    Seq(
      resolvers += ScalaToolsReleases,
      resolvers += ScalaToolsSnapshots,
      resolvers += "repo.novus snaps" at "http://repo.novus.com/snapshots/",
      libraryDependencies += scalatest,
      libraryDependencies += specs,
      libraryDependencies += unfiltered_spec,
//      libraryDependencies += antiXML,
      libraryDependencies += dispatch,
      libraryDependencies += unfiltered_filter,
      libraryDependencies += unfiltered_jetty,
      libraryDependencies += unfiltered_scalate,
      libraryDependencies += unfiltered_json,
      libraryDependencies += slf4jSimple,
      libraryDependencies += salat
    )

  lazy val virtual_trainer = Project(
    id = "virtualtrainer",
    base = file("."),
    settings = buildSettings ++ virtual_trainer_settings ++ sbtassembly.Plugin.assemblySettings
  )
  


}

