var Const = {
  distance : { km : 1, mile : 0.621371192 },
  height   : { cm : 1, inch : 0.393700787 },
  weight   : { kg : 1, lb   : 2.20462262 },
  
  // wait for at least 2 consecutive GPS data before declaring the GPS ready
  gps_max_errors : 2
  
};

var Util = {
  
  // Web Storage handling

  store : {
    enabled : (function() {
      try {
       return !!window.localStorage.getItem;
      } catch (e) {
       return false;
      }
    })(),
    set : function (key, value) {
      try {
        window.localStorage.setItem(key, JSON.stringify(value));
        return true;
      } catch (e) { };
      return false;
    },
    get : function (key) {
      try {
        var value = window.localStorage.getItem(key);
        return ((value != null)? JSON.parse(value) : null);
      } catch (e) { };
      return null;
    }
  },

  htmlLog : function (msg) {
    var element = document.getElementById("log");
    if (element != null) {
      if (element.textContent) element.textContent = msg;
      if (element.innerHTML) element.innerHTML = msg;
    }
  },

  log : function (msg) {
    try {
 //     console.log(msg);
      Util.htmlLog(msg);
    } catch (e) {
      Util.htmlLog(msg);
    }
  },

  // pretty formatting of numbers and time
  
  formatNumber : function (n) {
    return (n < 10)? "0" + n : n;
  },

  formatTime : function (time) {
    t = parseInt(time);
    hours = parseInt(t / 3600);
    minutes = parseInt(t / 60);
    seconds = t - ((hours * 3600) + (minutes * 60));
    return ((hours > 0)?  Util.formatNumber(hours) + ":": "") + Util.formatNumber(minutes) + ":" + Util.formatNumber(seconds);
  },
  
  formatTime2 : function (timestamp) {
  	date = new Date(timestamp);
  	return date.getHours() + ':' + date.getMinutes() + ':' + date.getSeconds();
  },
  
  formatDate : function (timestamp) {
  	date = new Date(timestamp);
  	return (parseInt(date.getMonth()) + 1).toString() + '/' + date.getDate() + '/' + date.getFullYear();
  },
  
  formatDistance : function (meters) {
  	return Math.round(meters/10)/100 + ' km';
  },


  // Geolocation related functions

  geo : {
  
    // calculate the distance in km between two GPS locations
    calculateDistance : function (lat1, lon1, lat2, lon2) {
      var dLat = (lat2-lat1) * Math.PI / 180;
      var dLon = (lon2-lon1) * Math.PI / 180;
      var lat1 = lat1  * Math.PI / 180;
      var lat2 = lat2 * Math.PI / 180;
      var a = Math.sin(dLat/2) * Math.sin(dLat/2) +
        Math.sin(dLon/2) * Math.sin(dLon/2) * Math.cos(lat1) * Math.cos(lat2);
      // 6371 km is average earth radius 
      return 6371 * 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a));      
    },
    
    // calculate the distance in km between several GPS locations
    calculateDistanceCoordinates : function (events) {
      var result = 0;
      if (events.length > 0) {
        var i = 1;
        while (i < events.length) {
          if (!!events[i-1].c && !!events[i].c) {
            var position1 = events[i-1].c;
            var position2 = events[i].c;
            result += Util.geo.calculateDistance(position1[1], position1[0], position2[1], position2[0]);
          }
          i++;
        }
      }
      return result;
    },
    
    // calculate the total duration between geolocation coordinates
    calculateDurationCoordinates : function (events) {
      var result = 0;
      if (events.length > 0) {
        var i = 1;
        while (i < events.length) {
          if (!!events[i-1].c && !!events[i].c) {
            result += (events[i].t - events[i-1].t);
          }
          i++;
        }
      }
      return result;
    }
  },
  
  // send information to the server as JSON
  // uri is a URI
  // resource is { contentType : string, data : string }
  // successHandler is function(XMLHttpRequest)
  // errorHandler is function(XMLHttpRequest || Exception)
  post : function (uri, resource, successHandler, errorHandler) {
  	
    var xhr = new XMLHttpRequest;
    var _emptyFunction = function() { };
    var successHandler = (!!successHandler)? successHandler : _emptyFunction;
    var errorHandler = (!!errorHandler)? errorHandler : _emptyFunction;
    try {
    	xhr.open("POST", uri, true);
    	xhr.onreadystatechange = function() {
        	if (xhr.readyState == 4) {
            	if (xhr.status === 201 || xhr.status === 200) {
	        		Util.log("XHR HTTP POST successfull " + xhr.status);
       		    	successHandler(xhr);
            	} else {
            		Util.log("[ERROR] XHR HTTP POST failed " + xhr.status);
    	        	errorHandler(xhr);      
            	}
        	} else {
            	Util.log("xhr readyState is " + xhr.readyState);
        	}
    	};
   		xhr.setRequestHeader("Content-Type", (!!resource.contentType)? resource.contentType : "application/octet-stream");
    	xhr.setRequestHeader("Content-Length", resource.data.length);
    	xhr.setRequestHeader("Accept", "text/plain");
    	if (!!resource.userId && !!resource.passwd) {
    		// we do basic auth
    		xhr.setRequestHeader("Authorization",
    							B64.encode('Basic ' + resource.userId + ':' + resource.passwd));
    	}
    	xhr.send(resource.data);
    } catch (e) {
      Util.log("Failed to send to server " + e);
      errorHandler(e); 
    }
  },

 // send information to the server as JSON
  postAsJSON : function (uri, resource, successHandler, errorHandler) {
  	try {
  		resource.contentType = "application/json";
  		resource.data = JSON.stringify(resource.data);
    	Util.post(uri, resource, successHandler, errorHandler);
   	} catch (e) {
   		errorHandler(e);
   	}
  },
  
  request : function (uri, body, callback, async) {
  	body = body ? body : null;
    callback = callback ? callback : null;
    //async = typeof(async) != 'undefined' ? async : false;
    async = !!async;
  	var http = new XMLHttpRequest();
	http.open('GET', uri, async);
	http.onreadystatechange = function () {
		if(http.readyState == 4 && http.status == 200){
			if (callback != null)
				callback(http.responseText);
		}
	};
	http.send(body);
	if (async == false)
		return http.responseText;
  },
  
  requestJSON : function (uri, body, callback, async) {
  	return JSON.parse(Util.request(uri, body, callback, async));
  }
  
};
