package org.w3.virtualtrainer

import org.w3.virtualtrainer.util.ResourceManager.fromClasspath

import unfiltered.request._
import unfiltered.response._
import unfiltered.scalate._
import unfiltered.jetty._

import org.w3.virtualtrainer.model._
import com.novus.salat._
import com.novus.salat.global._
import com.mongodb.casbah.Imports._

import org.fusesource.scalate.TemplateEngine
import org.slf4j.{Logger, LoggerFactory}

object VirtualTrainerApp {
  
  val logger:Logger = LoggerFactory.getLogger(this.getClass)

  val echo = unfiltered.filter.Planify {
    case Path(Seg(p :: Nil)) => ResponseString(p)
  }

  val engine = {
      val templates = fromClasspath("templates/")
      logger.debug("Template directory extracted at " + templates.getAbsolutePath)
      val templateDirs = List(templates)
      val scalateMode = "production"
      new TemplateEngine(templateDirs, scalateMode)
  }

  val tracker = unfiltered.filter.Planify {
    case req @ Path(Seg("tracker" :: Nil)) =>
      Ok ~> ContentType("application/xhtml+xml") ~> Scalate(req, "geolocation.ssp")(engine)
  }
  
  val app = unfiltered.filter.Planify {
    case req @ Path(Seg("app" :: Nil)) =>
      Ok ~> ContentType("application/xhtml+xml") ~> Scalate(req, "app.ssp")(engine)
  }
  
  val workoutAPI = unfiltered.filter.Planify {
    case req @ POST(Path(Seg("user" :: user :: "workout" :: Nil))) & Host(host) => {
      val json:MongoDBObject = com.mongodb.util.JSON.parse(Body.string(req)).asInstanceOf[com.mongodb.DBObject]
      json += "user" -> user
      val workout = grater[Workout].asObject(json)
      WorkoutDAO.insert(workout) match {
        case Some(id) => {
          val path = "%s://%s/user/%s/workout/%s" format (req.underlying.getScheme, host, user, id.toString)
          Created ~> Location(path) ~> ResponseString("object available at %s" format path)
        }
        case None => {
          InternalServerError ~> ResponseString("couldn't create object")
        }
      }
    }
    case req @ Path(Seg("user" :: user :: "workout" :: id :: Nil)) => req match {
      case GET(_) => {
        val realId = id.replaceAll(".json$", "").replaceAll(".gpx$", "")
        WorkoutDAO.findOneByID(realId) match {
          case Some(workout) if id endsWith ".gpx" =>
            Ok ~> ContentType("application/xml") ~> Scalate(req, "gpx.ssp", "w" -> workout)(engine)
          case Some(workout) if (id endsWith ".json") || (id == realId) =>
            Ok ~> ContentType("application/json") ~> ResponseString(workout.asJSON)
          case _ => NotFound
        }
      }
      case _ => MethodNotAllowed ~> Allow("GET") ~> ResponseString("MethodNotAllowed...")
    }
    case req @ Path(Seg("user" :: user :: "workouts" :: Nil)) => req match {
      case GET(_) & Host(host) => {
        def format(workout:Workout) = {
          import workout.{user => u, _}
          val uri = "%s://%s/user/%s/workout/%s" format (req.underlying.getScheme, host, user, id)
          """{ "activity": "%s", "date": %d, "duration": %d, "distance": %f, "json": "%s", "gpx": "%s" }""" format (activity, date, distance, 12.0, uri+".json", uri+".gpx")
        }
        val jsonResponse = WorkoutDAO.getAllWorkoutsByUser(user) map { w => format(w) } mkString ("[ ", ", ", " ]")
        Ok ~> ContentType("application/json") ~> ResponseString(jsonResponse)
      }
      case _ => MethodNotAllowed ~> Allow("GET") ~> ResponseString("MethodNotAllowed...")
    }
  }

}

import VirtualTrainerApp._
import org.w3.virtualtrainer.util.RequestLogger

object VirtualTrainerAppMain {

  val logger:Logger = LoggerFactory.getLogger(this.getClass)

  /**
   * 
   */
  def main(args: Array[String]) {
    
    val logger:Logger = LoggerFactory.getLogger(this.getClass)

    val port = args.toList.headOption map { _.toInt } getOrElse 2719
    
    unfiltered.jetty.Http(port)
    .filter(new RequestLogger {})
    .context("/scripts"){ ctx:ContextBuilder =>
      ctx.resources(fromClasspath("scripts/").toURI.toURL)
    }.context("/styles"){ ctx:ContextBuilder =>
      ctx.resources(fromClasspath("styles/").toURI.toURL)
    }.filter(tracker).filter(app).filter(workoutAPI).filter(echo).run()

    
  }

}
